function [tsave,ysave] = RKF45(f,y0,t0,T,h0,atol,rtol)

% function RKF45
%
% Zweck: integriere die gewoehnliche Diff.-Gleichung erster Ordnung
%
%          dy/dt = f(t,y(t))
%
%        mit einem Runge-Kutta-Fehlberg Verfahren basierend auf 
%        eingebetteten Runge-Kutta Verfahren der Ordnung 4 und 5
%
% Parameters: f    ... rechte Seite f(t,y(t)) der gew. Diff.-Gl.
%             t0,T ... Start- und End-Zeit
%             y0   ... Anfagswert
%             h0   ... Anfangs-Schrittweite
%             atol ... Absolute Toleranz
%
% Returns: t ... Zeiten
%          y ... approx. Loesung zu Zeiten t
%
%

% Initialisiere Zeit- und Loesungs-Vektor
y = y0;
t = t0;
h = h0;

% Parameter
p       = 4;
hmin    = 10*eps; % kleinste zugelassene Schrittweite
fac     = 0.9;    % Sicherheits-Faktor
facmin  = 0.5;    % kleinste zugelassene Schrittweiten-Verkleinerung
facmax  = 1.5;    % groesste zugelassene Schrittweiten-Vergroesserung
epsilon = atol + norm(y0)*rtol; % Initialisiere Fehlerschaetzer

tsave = t0;
ysave = y0;

while t < T
    tol = atol + norm(y)*rtol;
    h   = h*min(facmax,max(facmin,fac*(tol/epsilon)^(1./(p + 1.))));

    if ( h < hmin )
        fprintf('h too small\n')
        break
    end
    h = min(h, T - t);
    
    k1 = f(t,y);
    k2 = f(t + h/4,y + h*k1/4);
    k3 = f(t + 3*h/8,y + h*(3*k1 + 9*k2)/32);
    k4 = f(t + 12*h/13,y + h*(1932*k1 - 7200*k2 + 7296*k3)/2197);
    k5 = f(t + h,y + h*(439*k1/216 -8*k2 +3680*k3/513 - 845*k4/4104));
    k6 = f(t + h/2,y + h*(-8*k1/27 + 2*k2 -3544*k3/2565+1859*k4/4104 - 11*k5/40));

    ytemp_4 = y + h*(25*k1/216 + 1408*k3/2565 + 2197*k4/4104 - 1*k5/5);
    ytemp_5 = y + h*(16*k1/135 + 6656*k3/12825 + 28561*k4/56430 - 9*k5/50 + 2*k6/55);

    epsilon = norm(ytemp_5 - ytemp_4);

    if epsilon < tol
      t = t + h;
      y = ytemp_5;

      tsave = [tsave, t];
      ysave = [ysave, y];
    end
end
